@@ -12,21 +12,22 @@ import shlex
 import subprocess  # nosec B404:blacklist import_subprocess
 from functools import wraps
 from getpass import getpass
-from typing import List, Dict, Any, Optional, Callable
+from typing import List, Dict, Any, Optional, Callable, Type, Union, TypeVar, cast
 
 from .progress import CliProgress
-from .. import VERSION_WITH_DEPS
-from ..utils.config import Config, load_config, config_to_dict
 from .cli_builder import (
     rename,
     return_to_stdout,
     partial,
     lazy_partial,
-    Subcommands,
+    CliWithSubcommands,
     Subcommand,
     SubcommandGroup,
     decorate,
 )
+from ..core.secret import Secret
+from .. import VERSION_WITH_DEPS
+from ..utils.config import Config, load_config, config_to_dict
 from ..utils.log import (
     log_to_stream,
     log_to_rotating_file,
@@ -37,13 +38,19 @@ from ..utils.error_handling import (
     log_exceptions,
     error_report_hint_at_exit,
 )
+from ..utils.progress import ProgressInterface
 from ..workflows.config import create as create_config
 from ..workflows.transfer import transfer as workflows_transfer
 from ..workflows.decrypt import decrypt as workflows_decrypt
 from ..workflows.encrypt import encrypt as workflows_encrypt
 from ..workflows.request_sigs import request_sigs
 from ..workflows.upload_keys import upload_keys
-from ..protocols import Protocol, parse_protocol, __all__ as available_protocols, sftp
+from ..protocols import (
+    sftp,
+    liquid_files,
+    parse_protocol,
+    __all__ as available_protocols,
+)
 from ..core.versioncheck import check_version
 from ..core.error import UserError
 from ..core.filesystem import OutOfSpaceError
@@ -52,24 +59,35 @@ from .. import URL_READTHEDOCS, URL_GITLAB_ISSUES
 logger = create_logger(__name__)
 
 
-def parse_dict(s):
-    return json.loads(s)
+def parse_dict(s: str) -> Dict[str, Any]:
+    """Convert/deserialize a JSON formatted string to a dict object."""
+
+    return dict(json.loads(s))
 
 
-def parse_protocol_args(s):
+def parse_protocol_args(s: str) -> Dict[str, Any]:
+    """Convert/deserialize a string containing SFTP protocol values to a
+    dictionary object.
+
+    Example of input string:
+    {"host": "sftp.example.com", "username": "sftp",
+    "destination_dir": "upload", "pkey": "~/.ssh/id_rsa", "pkey_password": ""}
+    """
     args = parse_dict(s)
     not_provided = object()
     pw = args.get("pkey_password", not_provided)
     if pw is None:  # pw is provided and is None
-        args["pkey_password"] = getpass("Please enter your ssh private key password:")
+        args["pkey_password"] = Secret(
+            getpass("Please enter your ssh private key password:")
+        )
     return args
 
 
-def two_factor_cli_prompt():
-    return input("Verification code: ")
+def two_factor_cli_prompt() -> str:
+    return input("Please enter 2FA verification code: ")
 
 
-def get_passphrase_from_cmd(passphrase_cmd: str) -> str:
+def get_passphrase_from_cmd(passphrase_cmd: str) -> Secret[str]:
     try:
         # B603:subprocess_without_shell_equals_true
         # Suppress the warning, user can easily execute the same command
@@ -80,31 +98,34 @@ def get_passphrase_from_cmd(passphrase_cmd: str) -> str:
             stderr=subprocess.PIPE,
             check=True,
         )
-        return proc.stdout.decode().strip()
+        return Secret(proc.stdout.decode().strip())
     except subprocess.CalledProcessError as e:
         raise UserError(
-            "Failed to read passphrase from "
-            f"'{passphrase_cmd}'. "
+            f"Failed to read passphrase from '{passphrase_cmd}'. "
             f"{e.stderr.decode().strip()}"
         ) from e
 
 
-def get_passphrase_from_cmd_or_prompt(msg: str, passphrase_cmd: Optional[str]) -> str:
+def get_passphrase_from_cmd_or_prompt(
+    msg: str, passphrase_cmd: Optional[str]
+) -> Secret[str]:
     if passphrase_cmd:
         return get_passphrase_from_cmd(passphrase_cmd)
-    return getpass(msg)
+    return Secret(getpass(msg))
 
 
 @wraps(workflows_encrypt)
 def encrypt(
-    *args,
+    *files: str,
     config: Config,
     dry_run: bool,
     passphrase_cmd: Optional[str],
     dtr_id: Optional[int] = None,
     verify_dtr: Optional[bool] = None,
-    **kwargs,
-):
+    **kwargs: Any,
+) -> Optional[str]:
+    """Wrapper for the main function of the encrypt workflow."""
+
     if verify_dtr is None:
         verify_dtr = not config.offline and dtr_id is not None
     if config.offline and verify_dtr:
@@ -119,7 +140,7 @@ def encrypt(
         kwargs["passphrase"] = None
     try:
         return workflows_encrypt(
-            *args,
+            *files,
             dtr_id=dtr_id,
             verify_dtr=verify_dtr,
             config=config,
@@ -131,90 +152,90 @@ def encrypt(
 
 
 @wraps(workflows_decrypt)
-def decrypt(*args, dry_run: bool, passphrase_cmd: Optional[str], **kwargs):
+def decrypt(
+    *files: str, dry_run: bool, passphrase_cmd: Optional[str], **kwargs: Any
+) -> None:
     if dry_run:
         kwargs["passphrase"] = None
     else:
         kwargs["passphrase"] = get_passphrase_from_cmd_or_prompt(
             "Please enter your gpg private key password:", passphrase_cmd
         )
-    return workflows_decrypt(*args, dry_run=dry_run, **kwargs)
+    workflows_decrypt(*files, dry_run=dry_run, **kwargs)
 
 
 def transfer(
     files: List[str],
     *,
-    connection: str = None,
-    two_factor_callback,
+    connection: Optional[str] = None,
+    two_factor_callback: Callable[[], str],
     passphrase_cmd: Optional[str] = None,
     config: Config,
-    protocol=None,
-    protocol_args: Dict[str, Any] = None,
+    protocol: Optional[Type[Union[sftp.Protocol, liquid_files.Protocol]]] = None,
+    protocol_args: Optional[Dict[str, Any]] = None,
     dry_run: bool = False,
     verify_dtr: Optional[bool] = None,
-    progress=None,
-):
+    force: bool = False,
+    progress: Optional[ProgressInterface] = None,
+) -> None:
     if connection is not None:
         if protocol is not None:
             raise UserError(
-                "Arguments 'protocol' and 'connection' " "cannot be given together"
+                "Arguments 'protocol' and 'connection' cannot be given together"
             )
-        connection_obj = config.connections[connection]
-        protocol = parse_protocol(connection_obj.protocol)
-        if protocol_args is None:
-            protocol_args = {}
-        protocol_args = {**connection_obj.parameters, **protocol_args}
-    if protocol is None or protocol_args is None:
-        raise UserError(
-            "Either 'protocol' together with 'protocol_args' "
-            "or 'connection' "
-            "has to be given as an argument"
-        )
-    specific_protocol_args = {
-        sftp.Protocol: {"two_factor_callback": two_factor_callback}
-    }
-    try:
-        protocol_instance: Protocol = protocol(
-            **protocol_args, **specific_protocol_args.get(protocol, {})
-        )
-    except TypeError as e:
-        raise UserError(format(e).replace("__init__", protocol.__module__)) from None
-    for pw_arg in protocol_instance.required_password_args():
-        if protocol_args.get(pw_arg) is None:
-            setattr(
-                protocol_instance,
-                pw_arg,
-                get_passphrase_from_cmd_or_prompt(
-                    f"Please enter the password for the argument `{pw_arg}`:",
-                    passphrase_cmd,
-                ),
+        protocol_instance = config.connections[connection]
+    else:
+        if protocol is None or protocol_args is None:
+            raise UserError(
+                "Missing argument: either 'protocol' and 'protocol_args', "
+                "or 'connection' must be given."
             )
-    return workflows_transfer(
+        try:
+            protocol_instance = protocol(**protocol_args)
+        except TypeError as e:
+            raise UserError(
+                format(e).replace("__init__", protocol.__module__)
+            ) from None
+        for pw_arg in protocol_instance.required_password_args():
+            if protocol_args.get(pw_arg) is None:
+                setattr(
+                    protocol_instance,
+                    pw_arg,
+                    get_passphrase_from_cmd_or_prompt(
+                        f"Please enter the password for the argument `{pw_arg}`:",
+                        passphrase_cmd,
+                    ),
+                )
+    workflows_transfer(
         files,
         protocol=protocol_instance,
         config=config,
         dry_run=dry_run,
         verify_dtr=verify_dtr,
+        verify_pkg_name=False if force else config.verify_package_name,
+        pkg_name_suffix=config.package_name_suffix,
         progress=progress,
+        two_factor_callback=two_factor_callback,
     )
 
 
-def load_config_check():
+def load_config_and_check_app_is_up_to_date() -> Config:
+    """Load the config file and perform a check to see whether the user's
+    local instance of the application is up-to-date.
+    """
     cfg = load_config()
     if not cfg.offline and cfg.check_version:
         check_version(cfg.repo_url)
     return cfg
 
 
-class Cli(Subcommands):
+class Cli(CliWithSubcommands):
     description = __doc__.format(URL_READTHEDOCS, URL_GITLAB_ISSUES)
     version = VERSION_WITH_DEPS
-    config = load_config_check()
+    config = load_config_and_check_app_is_up_to_date()
     passphrase_override = dict(
-        help="Instead of asking for passphrase, "
-        "get it from an external command "
-        "(passphrase must be returned to the "
-        "standard output).",
+        help="Instead of asking for passphrase, get it from an external command "
+        "(passphrase must be returned to the standard output).",
         name="passphrase-cmd",
         dest="passphrase_cmd",
     )
@@ -222,7 +243,11 @@ class Cli(Subcommands):
         help="Perform checks on the input data, without running the actual command."
     )
     verify_dtr_override = dict(
-        help="Verify DTR ID against online database.", default=None
+        help="Verify DTR ID against an online database (portal). By default, "
+        "this check is performed if a DTR ID is provided or present in the "
+        "data package's metadata file, and skipped otherwise. Passing this "
+        "flag (or its disabling counterpart) will force (or disable) DTR ID "
+        "verification."
     )
     subcommands = (
         Subcommand(
@@ -302,6 +327,11 @@ class Cli(Subcommands):
                     "protocol_args option"
                 ),
                 "passphrase_cmd": dict(help=passphrase_override["help"]),
+                "force": dict(
+                    help="Ignore errors about package names not following the"
+                    "package name convention.",
+                    alias="-f",
+                ),
                 "dry_run": dry_run_override,
                 "verify_dtr": verify_dtr_override,
             },
@@ -342,12 +372,14 @@ class Cli(Subcommands):
     )
 
 
-def when(
-    condition: bool, decorator: Callable[[Callable], Callable]
-) -> Callable[[Callable], Callable]:
+# F is used as generic type for functions (Callable).
+F = TypeVar("F", bound=Callable[..., Any])
+
+
+def when(condition: bool, decorator: F) -> F:
     if condition:
         return decorator
-    return lambda x: x
+    return cast(F, lambda x: x)
 
 
 log_to_rotating_file(
@@ -359,7 +391,7 @@ log_to_stream()
 @exit_on_exception
 @when(Cli.config.error_reports, error_report_hint_at_exit(Cli.config))
 @log_exceptions(logger)
-def run():
+def run() -> int:
     if Cli():
         return 0
     return 1
