@@ -1,18 +1,19 @@
 from functools import partial
 from pathlib import Path
-from typing import Callable, Optional, Tuple, Sequence, Union
+from typing import Callable, Optional, Tuple, Sequence, Dict, Union, Any, cast
 
 from .model import AppData
 from .parallel import run_thread
 from .pyside import QtCore, QtWidgets, QAction, QtGui, open_window
 from .. import APP_NAME_SHORT
 from ..utils.progress import ProgressInterface
+from ..core.secret import Secret
 
 EXTRA_HEIGHT = 1
 MACOS_TOOLTIP_BG_COLOR = "#ffffca"
 
 
-def is_macos():
+def is_macos() -> bool:
     return QtCore.QSysInfo.productType() in ("osx", "macos")
 
 
@@ -33,24 +34,27 @@ class LineEdit(QtWidgets.QLineEdit):
 class ToolBar(QtWidgets.QToolBar):
     def __init__(self, title: str, parent: Optional[QtWidgets.QWidget] = None):
         super().__init__(title, parent)
-        # Remove the ugly gradient from the macos toolbar
+        # Remove the ugly gradient from the macos toolbar.
         if is_macos():
             self.setStyleSheet("background: transparent;border-width: 0px;")
         self.setIconSize(QtCore.QSize(20, 20))
 
 
 class MandatoryLabel(QtWidgets.QLabel):
-    """A label extension which appends a '*' to label's end to mark the field as required."""
+    """A label extension which appends a '*' at the end of the label to mark
+    the field as required.
+    """
 
     def __init__(self, label: str):
         super().__init__(label + " <sup><font color='red'>*</font></sup>")
 
 
 class SelectionButton(QtWidgets.QPushButton):
-    """A push button extension which connects this button to given selection model.
+    """A push button extension which connects this button to given selection
+    model.
 
-    The button is disabled by default. And gets enabled when the selection has, at least,
-    one selected row.
+    The button is disabled by default. And gets enabled when the selection has,
+    at least, one selected row.
     """
 
     def __init__(self, label: str, selection_model: QtCore.QItemSelectionModel):
@@ -59,49 +63,53 @@ class SelectionButton(QtWidgets.QPushButton):
         self._selection_model = selection_model
         selection_model.selectionChanged.connect(self.selection_changed)
 
-    def selection_changed(self):
-        # Following works better than using 'QItemSelection', especially in cases
-        # where multiple selection is possible
+    def selection_changed(self) -> None:
+        # Following works better than using 'QItemSelection', especially in
+        # cases where multiple selection is possible
         self.setEnabled(bool(len(self._selection_model.selectedRows())))
 
 
 class SelectionAction(QAction):
-    """An action extension which connects this action to given selection model.
+    """An action extension which connects this action to a selection model.
 
-    The action is disabled by default. And gets enabled when the selection has, at least,
-    one selected row.
+    The action is disabled by default. And gets enabled when the selection has,
+    at least, one selected row.
     """
 
-    def __init__(self, *args, selection_model: QtCore.QItemSelectionModel):
+    def __init__(self, *args: Any, selection_model: QtCore.QItemSelectionModel):
         super().__init__(*args)
         self.setEnabled(False)
         self._selection_model = selection_model
         selection_model.selectionChanged.connect(self.selection_changed)
 
-    def selection_changed(self):
-        # Following works better than using 'QItemSelection', especially in cases
-        # where multiple selection is possible
+    def selection_changed(self) -> None:
+        # Following works better than using 'QItemSelection', especially in
+        # cases where multiple selection is possible.
         self.setEnabled(bool(len(self._selection_model.selectedRows())))
 
 
 class GuiProgress(QtCore.QObject, ProgressInterface):
-    updated = QtCore.Signal(int)
+    # NOTE: the casting below is needed but it's not clear whether this is
+    #       because i) we are generating the SignalInstance object improperly,
+    #       or ii) PySide has implemented this badly and the cast to path their
+    #       implementation.
+    updated = cast(QtCore.SignalInstance, QtCore.Signal(int))
 
-    def __init__(self, update_callback: Callable):
+    def __init__(self, update_callback: Callable[..., None]):
         super().__init__()
-        self.n = 0
-        self.updated.connect(update_callback)  # type: ignore
+        self.n: float = 0
+        self.updated.connect(update_callback)
 
-    def update(self, completed_fraction):
+    def update(self, completed_fraction: float) -> None:
         self.n = completed_fraction
         self.updated.emit(round(completed_fraction * 100, 0))
 
-    def get_completed_fraction(self):
+    def get_completed_fraction(self) -> float:
         return self.n
 
 
 class ConsoleWidget(QtWidgets.QGroupBox):
-    def __init__(self, title, parent):
+    def __init__(self, title: str, parent: QtWidgets.QWidget):
         super().__init__(title, parent)
         self.textbox = QtWidgets.QTextEdit()
         self.textbox.setReadOnly(True)
@@ -112,7 +120,9 @@ class ConsoleWidget(QtWidgets.QGroupBox):
         btn_clear_console.clicked.connect(self.clear)
         layout = QtWidgets.QHBoxLayout(self)
         layout.addWidget(self.textbox)
-        layout.addWidget(btn_clear_console, alignment=QtCore.Qt.AlignTop)
+        layout.addWidget(
+            btn_clear_console, alignment=cast(QtCore.Qt.Alignment, QtCore.Qt.AlignTop)
+        )
 
     def clear(self) -> None:
         self.textbox.clear()
@@ -124,12 +134,18 @@ class ConsoleWidget(QtWidgets.QGroupBox):
         self.textbox.append(text)
 
 
-class PathInput:
-    """Path selection widget with a select button and a show path field."""
+class PathInput(QtWidgets.QWidget):
+    """Path selection widget with a select button and a 'show path' field."""
 
-    def __init__(self, directory=True, path: Optional[Path] = Path.home(), parent=None):
-        self.parent = parent
-        self.text = LineEdit(parent)
+    def __init__(
+        self,
+        *args: Any,
+        directory: bool = True,
+        path: Optional[Path] = Path.home(),
+        **kwargs: Any,
+    ):
+        super().__init__(*args, **kwargs)
+        self.text = LineEdit(self.parent())
         self.text.setReadOnly(True)
         self.btn = QtWidgets.QPushButton(
             QtGui.QIcon(f":icon/feather/{'folder' if directory else 'file'}.png"),
@@ -137,7 +153,8 @@ class PathInput:
         )
         self.btn.setToolTip("Change location")
         self.btn.clicked.connect(partial(self._update_location, directory))
-        # Additional action to clear the selected path
+
+        # Add additional button to allow clearing the selected path.
         self.btn_clear = QtWidgets.QPushButton(
             QtGui.QIcon(":icon/feather/slash.png"), ""
         )
@@ -145,28 +162,28 @@ class PathInput:
         self.btn_clear.clicked.connect(self._clear_location)
         self.update_path(path)
 
-    def update_path(self, path: Optional[Path]):
+    def update_path(self, path: Optional[Path]) -> None:
         self.path = path
         self.text.setText("" if path is None else str(path))
         self.text.editingFinished.emit()
 
-    def _update_location(self, directory: bool):
+    def _update_location(self, directory: bool) -> None:
         if self.path and self.path.exists():
             location = self.path if self.path.is_dir() else self.path.parent
         else:
             location = Path.home()
         if directory:
             new_path = QtWidgets.QFileDialog.getExistingDirectory(
-                self.parent, "Select Directory", str(location)
+                parent=self.parent(), caption="Select Directory", dir=str(location)
             )
         else:
             new_path, _ = QtWidgets.QFileDialog.getOpenFileName(
-                self.parent, "Select File", str(location)
+                parent=self.parent(), caption="Select File", dir=str(location)
             )
         if new_path:
             self.update_path(Path(new_path))
 
-    def _clear_location(self):
+    def _clear_location(self) -> None:
         self.update_path(None)
 
     @property
@@ -174,7 +191,7 @@ class PathInput:
         return self.text.statusTip()
 
     @status_tip.setter
-    def status_tip(self, msg: str):
+    def status_tip(self, msg: str) -> None:
         self.text.setStatusTip(msg)
 
     def on_path_change(self, fn: Callable[[Optional[Path]], None]) -> None:
@@ -182,11 +199,20 @@ class PathInput:
         self.text.editingFinished.connect(lambda: fn(self.path))
 
 
-class TabMixin:
-    def create_console(self):
-        self.console = ConsoleWidget("Console", self)
+class TabMixin(QtWidgets.QWidget):
+    """Adds two boxes to the layout of a QWidget (a Tab from the application):
+    - Box with buttons to run and test run a workflow.
+    - Box with a console.
 
-    def create_progress_bar(self):
+    This class is a "mixin" and is destined to be inherited by another class
+    that will provide the constructor.
+    """
+
+    def create_console(self) -> None:
+        self.console = ConsoleWidget(title="Console", parent=self)
+
+    def create_progress_bar(self) -> None:
+        # NOTE: same problem here as above.
         self.progress_bar = QtWidgets.QProgressBar(self)
 
     @staticmethod
@@ -195,26 +221,32 @@ class TabMixin:
         button.setEnabled(False)
         return button
 
-    def set_buttons_enabled(self, enabled: bool):
+    def set_buttons_enabled(self, enabled: bool) -> None:
         self.btn_run.setEnabled(enabled)
         self.btn_test.setEnabled(enabled)
 
-    def create_run_panel(self, panel_name: str, action: Callable, action_name: str):
+    def create_run_panel(
+        self, panel_name: str, action: Callable[..., Any], action_name: str
+    ) -> None:
         self.run_panel = QtWidgets.QGroupBox(panel_name)
         self.btn_test = TabMixin._create_disabled_button("Test")
-        # On pressed button, make sure that the focus switches to that button (Mac specific issue)
+
+        # On pressed button, make sure that the focus switches to that
+        # button (Mac specific issue)
         self.btn_test.pressed.connect(self.btn_test.setFocus)
         self.btn_test.clicked.connect(partial(action, dry_run=True))
         self.btn_run = TabMixin._create_disabled_button(action_name)
-        # On pressed button, make sure that the focus switches to that button (Mac specific issue)
+
+        # On pressed button, make sure that the focus switches to that
+        # button (Mac specific issue)
         self.btn_run.pressed.connect(self.btn_run.setFocus)
         self.btn_run.clicked.connect(action)
         hbox_layout(self.btn_test, self.btn_run, parent=self.run_panel)
 
-    def run_workflow_thread(self, f: Callable, **kwargs):
-        """Run a thread with predefined signals"""
+    def run_workflow_thread(self, f: Callable[..., Any], **kwargs: Any) -> None:
+        """Run a thread with predefined signals."""
         self.run_panel.setEnabled(False)
-        signals = dict(
+        signals: Dict[str, Callable[..., Any]] = dict(
             logging=self.console.write,
             error=lambda e: self.console.append(str(e[1])),
             finished=lambda: self.run_panel.setEnabled(True),
@@ -222,7 +254,9 @@ class TabMixin:
         run_thread(f, signals=signals, **kwargs)
 
 
-def get_text_input(parent, msg: str, password: bool = True) -> Optional[str]:
+def get_text_input(
+    parent: QtWidgets.QWidget, msg: str, password: bool = False
+) -> Optional[str]:
     dialog = QtWidgets.QDialog(parent)
     dialog.setWindowTitle(APP_NAME_SHORT)
     buttons = QtWidgets.QDialogButtonBox(QtWidgets.QDialogButtonBox.Cancel)
@@ -240,13 +274,18 @@ def get_text_input(parent, msg: str, password: bool = True) -> Optional[str]:
     return user_input.text()
 
 
+def get_pass_input(parent: QtWidgets.QWidget, msg: str) -> Optional[Secret[str]]:
+    raw_pw = get_text_input(parent, msg, password=True)
+    return None if raw_pw is None else Secret(raw_pw)
+
+
 def warning_callback(
     title: str, parent: Optional[QtWidgets.QWidget] = None
-) -> Callable:
+) -> Callable[[str], None]:
     msg_warn = warning_dialog(title, parent=parent)
 
-    def _show_warning(msg: str):
-        # add an extra line break so that text is easier to read in GUI.
+    def _show_warning(msg: str) -> None:
+        # Add an extra line break so that text is easier to read in GUI.
         msg_warn.setText(format(msg).replace("\n", "\n\n"))
         open_window(msg_warn)
 
@@ -264,7 +303,9 @@ def warning_dialog(
     return msg
 
 
-def show_warning(title: str, text: str, parent: Optional[QtWidgets.QWidget] = None):
+def show_warning(
+    title: str, text: str, parent: Optional[QtWidgets.QWidget] = None
+) -> None:
     warning_dialog(title, text, parent).show()
 
 
@@ -277,7 +318,7 @@ def create_verify_dtr_checkbox(
     if app_data.config.offline:
         verify_dtr.setEnabled(not app_data.config.offline)
         verify_dtr.setStatusTip(
-            "Verifying Transfer Request ID in offline mode is not possible"
+            "Verifying Transfer Request ID in offline mode is not possible."
         )
     verify_dtr.stateChanged.connect(
         lambda state: setattr(app_data, field_name, state == QtCore.Qt.Checked)
@@ -285,6 +326,21 @@ def create_verify_dtr_checkbox(
     return verify_dtr
 
 
+def create_verify_filename_checkbox(
+    app_data: AppData, field_name: str, parent: Optional[QtWidgets.QWidget] = None
+) -> QtWidgets.QCheckBox:
+    verify_filename = QtWidgets.QCheckBox("Verify package name", parent)
+    verify_filename.setStatusTip(
+        "Verify that the name of packages to transfer follows the package "
+        "naming convention."
+    )
+    verify_filename.setChecked(app_data.config.verify_package_name)
+    verify_filename.stateChanged.connect(
+        lambda state: setattr(app_data, field_name, state == QtCore.Qt.Checked)
+    )
+    return verify_filename
+
+
 def create_slider(
     minimum: int,
     maximum: int,
@@ -306,7 +362,7 @@ def create_slider(
     if show_ticks:
         slider.setTickPosition(QtWidgets.QSlider.TicksBelow)
 
-    def update_text(value: int):
+    def update_text(value: int) -> None:
         widget_value.setText(str(value))
 
     slider.valueChanged.connect(update_text)
@@ -317,18 +373,29 @@ def create_slider(
 
 
 class GridLayoutCell:
-    def __init__(self, widget, span: int = 1, align=None):
+    def __init__(
+        self,
+        widget: QtWidgets.QWidget,
+        span: int = 1,
+        align: Optional[QtCore.Qt.AlignmentFlag] = None,
+    ):
         self.widget = widget
         self.span = span
         self.align = align
 
 
-def layout_add(layout, child: Union[QtWidgets.QWidget, QtWidgets.QLayout]) -> Callable:
-    """Returns either layout.addLayout or layout.addWidget depending on the type of child"""
+def layout_add(
+    layout: QtWidgets.QLayout, child: Union[QtWidgets.QWidget, QtWidgets.QLayout]
+) -> Callable[..., Any]:
+    """Returns either layout.addLayout or layout.addWidget depending on the
+    type of child.
+    """
+    add_layout_function: Callable[..., Any] = layout.addLayout
+    add_widget_function: Callable[..., Any] = layout.addWidget
     return (
-        layout.addLayout
+        add_layout_function
         if isinstance(child, (QtWidgets.QVBoxLayout, QtWidgets.QHBoxLayout))
-        else layout.addWidget
+        else add_widget_function
     )
 
 
@@ -362,9 +429,9 @@ def box_layout(
     *widgets: QtWidgets.QWidget,
     parent: Optional[QtWidgets.QWidget] = None,
     LayoutClass: type,
-):
+) -> QtWidgets.QVBoxLayout:
     parent_args = () if parent is None else (parent,)
-    layout = LayoutClass(*parent_args)
+    layout: QtWidgets.QVBoxLayout = LayoutClass(*parent_args)
     for widget in widgets:
         add_method = layout_add(layout, widget)
         add_method(widget)
