@@ -1,10 +1,11 @@
 from functools import partial
+from typing import Any
 
 from .component import (
     TabMixin,
     PathInput,
     GuiProgress,
-    get_text_input,
+    get_pass_input,
     vbox_layout,
     hbox_layout,
 )
@@ -13,8 +14,10 @@ from .pyside import QtCore, QtWidgets
 from ..workflows import decrypt
 
 
-class DecryptTab(QtWidgets.QWidget, TabMixin):
-    def __init__(self, *args, **kwargs):
+class DecryptTab(TabMixin):
+    """Class that creates the "Decrypt" Tab of the GUI application."""
+
+    def __init__(self, *args: Any, **kwargs: Any):
         super().__init__(*args, **kwargs)
         self.app_data = self.parent().app_data
 
@@ -34,17 +37,17 @@ class DecryptTab(QtWidgets.QWidget, TabMixin):
             parent=self,
         )
 
-    def _enable_buttons(self):
+    def _enable_buttons(self) -> None:
         self.set_buttons_enabled(len(self.app_data.decrypt_files) > 0)
 
-    def create_files_panel(self):
+    def create_files_panel(self) -> QtWidgets.QGroupBox:
         box = ArchiveOnlyFileSelectionWidget(title="Files to decrypt", parent=self)
         box.file_list_model.layoutChanged.connect(
             lambda: setattr(self.app_data, "decrypt_files", box.get_list())
         )
         return box
 
-    def create_decrypt_options_panel(self):
+    def create_decrypt_options_panel(self) -> QtWidgets.QGroupBox:
         box = QtWidgets.QGroupBox("Output")
 
         decompress = QtWidgets.QCheckBox("Decompress", box)
@@ -68,9 +71,9 @@ class DecryptTab(QtWidgets.QWidget, TabMixin):
         vbox_layout(decompress, layout_output, parent=box)
         return box
 
-    def decrypt(self, dry_run=False):
+    def decrypt(self, dry_run: bool = False) -> None:
         if not dry_run:
-            pw = get_text_input(self, "Enter password for your GPG key")
+            pw = get_pass_input(self, "Enter password for your GPG key")
             if pw is None:
                 return
         else:
