@@ -1,11 +1,12 @@
 from functools import partial, wraps
+from typing import Any
 
 from .component import (
     TabMixin,
     PathInput,
     GuiProgress,
     create_slider,
-    get_text_input,
+    get_pass_input,
     SelectionButton,
     MandatoryLabel,
     create_verify_dtr_checkbox,
@@ -29,7 +30,7 @@ ignore_disk_space_error_label = "Ignore disk space error"
 
 
 @wraps(encrypt.encrypt)
-def encrypt_workflow(*args, **kwargs):
+def encrypt_workflow(*args: str, **kwargs: Any) -> None:
     try:
         encrypt.encrypt(*args, **kwargs)
     except OutOfSpaceError as e:
@@ -39,12 +40,12 @@ def encrypt_workflow(*args, **kwargs):
         ) from e
 
 
-class EncryptTab(QtWidgets.QWidget, TabMixin):
-    def __init__(self, *args, **kwargs):
+class EncryptTab(TabMixin):
+    """Class that creates the "Encrypt" Tab of the GUI application."""
+
+    def __init__(self, *args: Any, **kwargs: Any):
         super().__init__(*args, **kwargs)
-
         self.app_data = self.parent().app_data
-
         files_panel = self.create_files_panel()
         self.create_run_panel(
             "Package and encrypt data", self.encrypt, "Package && Encrypt"
@@ -63,20 +64,20 @@ class EncryptTab(QtWidgets.QWidget, TabMixin):
             parent=self,
         )
 
-    def _enable_buttons(self):
+    def _enable_buttons(self) -> None:
         self.set_buttons_enabled(
             len(self.app_data.encrypt_recipients) > 0
             and len(self.app_data.encrypt_files) > 0
         )
 
-    def create_files_panel(self):
+    def create_files_panel(self) -> QtWidgets.QGroupBox:
         box = DirectoryAndFileSelectionWidget(title="Data to encrypt", parent=self)
         box.file_list_model.layoutChanged.connect(
             lambda: setattr(self.app_data, "encrypt_files", box.get_list())
         )
         return box
 
-    def create_user_panel(self):
+    def create_user_panel(self) -> QtWidgets.QGroupBox:
         group_box = QtWidgets.QGroupBox("Data sender and recipients")
         sender_widget = QtWidgets.QComboBox()
         sender_widget.setModel(self.app_data.priv_keys_model)
@@ -109,32 +110,32 @@ class EncryptTab(QtWidgets.QWidget, TabMixin):
         recipients_btn_remove.setIcon(QtGui.QIcon(":icon/feather/user-minus.png"))
         recipients_btn_remove.setToolTip("Remove recipient from the list")
 
-        def update_sender(index):
+        def update_sender(index: int) -> None:
             self.app_data.encrypt_sender = (
                 ""
                 if index == -1
                 else self.app_data.priv_keys_model.get_value(index).fingerprint
             )
 
-        def update_recipients(index):
+        def update_recipients(index: QtCore.QModelIndex) -> None:
             self.app_data.encrypt_recipients = [x[2] for x in index.model().get_data()]
 
-        def add_recipient():
+        def add_recipient() -> None:
             key = recipients_input_view.model().get_value(
                 recipients_input_view.currentIndex()
             )
             row = (key.uids[0].full_name, key.uids[0].email, key.fingerprint)
             recipients_output_model.set_data(
-                set(recipients_output_model.get_data()) | set([row])
+                tuple(set(recipients_output_model.get_data()) | set([row]))
             )
 
-        def remove_recipient():
+        def remove_recipient() -> None:
             recipients_output_view.model().removeRows(
                 recipients_output_view.currentIndex().row(), 1
             )
             recipients_output_view.clearSelection()
 
-        # Connect actions
+        # Connect actions.
         recipients_btn_add.clicked.connect(add_recipient)
         recipients_btn_remove.clicked.connect(remove_recipient)
         recipients_output_model.dataChanged.connect(update_recipients)
@@ -162,7 +163,7 @@ class EncryptTab(QtWidgets.QWidget, TabMixin):
         )
         return group_box
 
-    def _create_sender_transferinfo_and_output_layout(self):
+    def _create_sender_transferinfo_and_output_layout(self) -> QtWidgets.QVBoxLayout:
         """Create a layout with 3 group boxes: 'Data sender and recipients',
         'Data transfer info' and 'Output'.
         """
@@ -171,9 +172,11 @@ class EncryptTab(QtWidgets.QWidget, TabMixin):
         )
         return hbox_layout(self.create_user_panel(), layout_right)
 
-    def _create_output_group_box(self):
+    def _create_output_group_box(self) -> QtWidgets.QGroupBox:
         """Create a group box with the output-related fields."""
+
         group_box = QtWidgets.QGroupBox("Output")
+
         # Create fields
         output_suffix = LineEdit()
         output_suffix.setText(self.app_data.encrypt_package_name_suffix)
@@ -185,10 +188,11 @@ class EncryptTab(QtWidgets.QWidget, TabMixin):
         )
         output_location = PathInput(path=self.app_data.encrypt_output_location)
         output_location.status_tip = "Destination folder for the encrypted package"
+
         # Add actions
         output_suffix.editingFinished.connect(
             lambda: setattr(
-                self.app_data, "encrypt_output_suffix", output_suffix.text()
+                self.app_data, "encrypt_package_name_suffix", output_suffix.text()
             )
         )
         output_location.on_path_change(
@@ -236,10 +240,12 @@ class EncryptTab(QtWidgets.QWidget, TabMixin):
         )
         return group_box
 
-    def _create_transferinfo_group_box(self):
+    def _create_transferinfo_group_box(self) -> QtWidgets.QGroupBox:
         """Create a group box with data transfer request (DTR) related fields."""
+
         group_box = QtWidgets.QGroupBox("Data transfer info")
-        # Create fields
+
+        # Create fields.
         transfer_id = LineEdit()
         transfer_id.setStatusTip("(optional) Data Transfer Request ID")
         transfer_id.setValidator(QtGui.QIntValidator(1, 10 ** 9))
@@ -251,8 +257,8 @@ class EncryptTab(QtWidgets.QWidget, TabMixin):
         purpose.addItems(("",) + tuple(x.name for x in Purpose))
         purpose.setCurrentText("")
 
-        # Add actions
-        def on_transfer_id_changed(text):
+        # Add actions.
+        def on_transfer_id_changed(text: str) -> None:
             self.app_data.encrypt_transfer_id = int(text) if text else None
 
         transfer_id.textChanged.connect(on_transfer_id_changed)
@@ -269,9 +275,9 @@ class EncryptTab(QtWidgets.QWidget, TabMixin):
         )
         return group_box
 
-    def encrypt(self, dry_run=False):
+    def encrypt(self, dry_run: bool = False) -> None:
         if not dry_run and self.app_data.config.sign_encrypted_data:
-            pw = get_text_input(self, "Enter password for your GPG key")
+            pw = get_pass_input(self, "Enter password for your GPG key")
             if pw is None:
                 return
         else:
@@ -288,7 +294,7 @@ class EncryptTab(QtWidgets.QWidget, TabMixin):
                 config=self.app_data.config,
                 passphrase=pw,
                 output=self.app_data.encrypt_output_location,
-                output_suffix=self.app_data.encrypt_output_suffix,
+                output_suffix=self.app_data.encrypt_package_name_suffix,
                 dry_run=dry_run,
                 compression_level=self.app_data.encrypt_compression_level,
                 purpose=self.app_data.encrypt_purpose,
