@@ -3,7 +3,8 @@ from typing import Optional
 from .model import AppData
 from .parallel import run_thread
 from .component import warning_callback
-from ..core.crypt import gpg, load_authority_key
+from ..core import gpg
+from ..core.crypt import load_authority_key
 
 
 def load_authority_key_threaded(app_data: AppData) -> None:
@@ -13,7 +14,7 @@ def load_authority_key_threaded(app_data: AppData) -> None:
     """
     # This function is responsible for taking the returned value from
     # load_authority_key() and assign it to the correct attribute of app_data.
-    def update_authority_key_in_app_data(key: Optional[gpg.Key]):
+    def update_authority_key_in_app_data(key: Optional[gpg.Key]) -> None:
         app_data.validation_authority_key = key
 
     show_warning = warning_callback("Certification Authority Key Loading Error")
