@@ -1,5 +1,5 @@
+from typing import Callable
 import warnings
-import copy
 
 from sett import (
     APP_NAME_LONG,
@@ -20,17 +20,20 @@ from .pyside import QtCore, QtGui, QtWidgets, QAction, open_window
 from .settings_tab import SettingsTab
 from .transfer_tab import TransferTab
 from ..core.versioncheck import check_version
-from ..utils.config import load_config, save_config, config_to_dict
+from ..utils.config import Config, load_config, save_config, config_to_dict
 
 QtCore.QThreadPool.globalInstance().setExpiryTimeout(-1)
 
 
 class MainWindow(QtWidgets.QMainWindow):
-    def __init__(self):
+    """Class that initializes the main QtWidget (window) of the application,
+    onto which all other components will be added.
+    """
+
+    def __init__(self) -> None:
         super().__init__()
         self.title = f"{APP_NAME_LONG} ({__version__})"
-        self.app_data = AppData(config=self.get_config())
-        self.config_on_disk = copy.deepcopy(self.app_data.config)
+        self.app_data = AppData(config=self.load_config_from_disk())
 
         self.setWindowTitle(self.title)
         self.setMinimumSize(QtCore.QSize(900, 900))
@@ -39,7 +42,8 @@ class MainWindow(QtWidgets.QMainWindow):
         self.add_status_bar()
         self.check_version()
 
-    def get_config(self):
+    def load_config_from_disk(self) -> Config:
+        """Load user config settings from the default config file."""
         with warnings.catch_warnings(record=True) as w:
             config = load_config()
             if w:
@@ -50,7 +54,7 @@ class MainWindow(QtWidgets.QMainWindow):
                 )
         return config
 
-    def add_tabs(self):
+    def add_tabs(self) -> None:
         tab_keys = KeysTab(self)  # FIXME created first to populate gpg keys
         tab_encrypt = EncryptTab(self)
         tab_decrypt = DecryptTab(self)
@@ -68,11 +72,11 @@ class MainWindow(QtWidgets.QMainWindow):
 
         self.setCentralWidget(tabs)
 
-    def add_status_bar(self):
+    def add_status_bar(self) -> None:
         self.status = QtWidgets.QStatusBar()
         self.setStatusBar(self.status)
 
-    def add_menu(self):
+    def add_menu(self) -> None:
         action_exit = QAction(QtGui.QIcon(":icon/feather/log-out.png"), "&Exit", self)
         action_exit.setShortcut(QtGui.QKeySequence("Ctrl+Q"))
         action_exit.setStatusTip("Exit application")
@@ -105,13 +109,19 @@ class MainWindow(QtWidgets.QMainWindow):
         menu_help.addAction(action_bug_report)
         menu_help.addAction(action_about)
 
-    def closeEvent(self, event):
-        if self.app_data.config != self.config_on_disk:
+    def closeEvent(self, event: QtGui.QCloseEvent) -> None:
+        """Code that is run when the application is closed by the user."""
+
+        # Compare the current in-memory config values to those present on disk
+        # (in the user's config file). If they differ, ask the user whether
+        # they want to save their config value modifications to disk.
+        if self.app_data.config != self.load_config_from_disk():
             reply = QtWidgets.QMessageBox.question(
                 self,
                 "Persist changed settings?",
-                "You made changes to settings you did not persist yet.\nDo you "
-                "want to permanently persist them to your sett configuration file?",
+                "You made changes to settings you did not persist yet.\n"
+                "Do you want to permanently persist them to your sett "
+                "configuration file?",
                 QtWidgets.QMessageBox.Yes | QtWidgets.QMessageBox.No,
                 QtWidgets.QMessageBox.No,
             )
@@ -131,11 +141,11 @@ class MainWindow(QtWidgets.QMainWindow):
             else:
                 event.ignore()
 
-    def check_version(self):
+    def check_version(self) -> None:
         if self.app_data.config.offline or not self.app_data.config.check_version:
             return
 
-        def get_warnings():
+        def get_warnings() -> str:
             with warnings.catch_warnings(record=True) as w:
                 check_version(self.app_data.config.repo_url, gui_formatting=True)
                 return "\n".join(format(warning.message) for warning in w)
@@ -143,10 +153,12 @@ class MainWindow(QtWidgets.QMainWindow):
         run_thread(
             get_warnings,
             f_kwargs=dict(report_config=self.app_data.config),
-            signals=dict(result=lambda x: x and show_warning(self.title, x, self)),
+            signals=dict(
+                result=lambda x: show_warning(self.title, x, self) if x else x
+            ),
         )
 
-    def show_about(self):
+    def show_about(self) -> None:
         msg = QtWidgets.QMessageBox(parent=self)
         msg.setWindowTitle(f"About {APP_NAME_SHORT}")
         msg.setIcon(QtWidgets.QMessageBox.Information)
@@ -167,7 +179,7 @@ class MainWindow(QtWidgets.QMainWindow):
         msg.show()
 
 
-def open_url(url: str):
+def open_url(url: str) -> Callable[[], None]:
     """Returns a function that will open the specified URL in the user's
     default browser when called. The returned function has no arguments.
 
@@ -175,7 +187,7 @@ def open_url(url: str):
     :returns: function that opens the specified URL.
     """
 
-    def open_url_template():
+    def open_url_template() -> None:
         if not QtGui.QDesktopServices.openUrl(QtCore.QUrl(url)):
             msg_warn = QtWidgets.QMessageBox()
             msg_warn.setWindowTitle("Warning")
