@@ -7,18 +7,26 @@ from typing import (
     List,
     Optional,
     Tuple,
+    Dict,
     Union,
     Sequence,
     Type,
+    cast,
 )
 
+from libbiomedit.lib.secret import Secret
+
 
 from .pyside import QtCore
-from .ui_model_bind import ClassWithListener, DictWithListener
-from ..core.crypt import gpg
+from .ui_model_bind import ClassWithListener
+from ..core import gpg
 from ..core.metadata import Purpose
 from ..utils.config import Config
-from ..protocols import Protocol, protocols, sftp
+from ..protocols import Protocol, sftp, liquid_files
+from .listener import ListenerWrap
+
+# Default value Qt.DisplayRole for index is not an int (upstream issue).
+default_display_role = cast(int, QtCore.Qt.DisplayRole)
 
 
 class TableModel(QtCore.QAbstractTableModel):
@@ -36,7 +44,7 @@ class TableModel(QtCore.QAbstractTableModel):
         return self._columns
 
     @columns.setter
-    def columns(self, values: Sequence[str]):
+    def columns(self, values: Sequence[str]) -> None:
         self._columns = tuple(values)
 
     def get_data(self) -> Tuple[Sequence[Any], ...]:
@@ -57,23 +65,27 @@ class TableModel(QtCore.QAbstractTableModel):
         return len(self._data) and len(self._data[0])
 
     def headerData(
-        self, section: int, orientation=QtCore.Qt.Horizontal, role=QtCore.Qt.DisplayRole
+        self,
+        section: int,
+        orientation: QtCore.Qt.Orientation = QtCore.Qt.Horizontal,
+        role: int = default_display_role,
     ) -> Optional[str]:
-        if orientation == QtCore.Qt.Horizontal and role == QtCore.Qt.DisplayRole:
+        if orientation == QtCore.Qt.Horizontal and role == default_display_role:
             return self._columns[section]
-        if orientation == QtCore.Qt.Vertical and role == QtCore.Qt.DisplayRole:
+        if orientation == QtCore.Qt.Vertical and role == default_display_role:
             return str(section)
         return None
 
-    # Default value Qt.DisplayRole for index is not an int (upstream issue):
-    def data(  # type: ignore
-        self, index: QtCore.QModelIndex, role: int
-    ) -> Optional[Hashable]:
-        if role == QtCore.Qt.DisplayRole:
+    def data(
+        self, index: QtCore.QModelIndex, role: int = default_display_role
+    ) -> Optional[Any]:
+        if role == default_display_role:
             return self._data[index.row()][index.column()]
         return None
 
-    def removeRows(self, row: int, count: int, _parent=QtCore.QModelIndex()) -> bool:
+    def removeRows(
+        self, row: int, count: int, _parent: QtCore.QModelIndex = QtCore.QModelIndex()
+    ) -> bool:
         if row > -1 and row + count <= self.rowCount():
             self.beginRemoveRows(QtCore.QModelIndex(), row, row + count - 1)
             self.set_data(self.get_data()[:row] + self.get_data()[row + count :])
@@ -86,7 +98,10 @@ class KeyValueListModel(QtCore.QAbstractListModel):
     """List model extension for key-value objects."""
 
     def __init__(
-        self, *args, data: Optional[Iterable[Tuple[Hashable, Any]]] = None, **kwargs
+        self,
+        *args: Any,
+        data: Optional[Iterable[Tuple[Hashable, Any]]] = None,
+        **kwargs: Any,
     ):
         super().__init__(*args, **kwargs)
         self.set_data(data or [])
@@ -99,7 +114,8 @@ class KeyValueListModel(QtCore.QAbstractListModel):
     # Default value Qt.DisplayRole for index is not an int (upstream issue):
     def data(self, index, role) -> Optional[Hashable]:  # type: ignore
         if role == QtCore.Qt.DisplayRole:
-            return self._keys[index.row()]
+            key: Hashable = self._keys[index.row()]
+            return key
         return None
 
     def rowCount(self, _parent: QtCore.QModelIndex = QtCore.QModelIndex()) -> int:
@@ -113,13 +129,24 @@ class KeyValueListModel(QtCore.QAbstractListModel):
         raise TypeError("Wrong index type for index.")
 
 
-def protocol_factory():
-    return {p: DictWithListener() for p in protocols}
+def protocol_factory() -> Dict[Type[Protocol], ListenerWrap]:
+    return {
+        sftp.Protocol: ListenerWrap(new_sftp_connection()),
+        liquid_files.Protocol: ListenerWrap(new_liquid_files_connection()),
+    }
+
+
+def new_sftp_connection() -> Protocol:
+    return sftp.Protocol(host="", username="", destination_dir="")
+
+
+def new_liquid_files_connection() -> Protocol:
+    return liquid_files.Protocol(host="", api_key=Secret(""))
 
 
 @dataclass
 class AppData(ClassWithListener):
-    """Settings that are specific to the sett GUI"""
+    """Settings that are specific to the sett GUI."""
 
     config: Config
 
@@ -128,7 +155,6 @@ class AppData(ClassWithListener):
     encrypt_transfer_id: Optional[str] = None
     encrypt_purpose: Optional[Purpose] = None
     encrypt_compression_level: int = 6
-    encrypt_output_suffix: str = ""
     encrypt_output_location: Path = field(default_factory=Path.home)
     encrypt_files: List[str] = field(default_factory=list)
     encrypt_verify_dtr: bool = True
@@ -140,9 +166,12 @@ class AppData(ClassWithListener):
     decrypt_files: List[str] = field(default_factory=list)
 
     transfer_protocol_type: Type[Protocol] = sftp.Protocol
-    transfer_protocol_args: dict = field(default_factory=protocol_factory)
+    transfer_protocol_args: Dict[Type[Protocol], ListenerWrap] = field(
+        default_factory=protocol_factory
+    )
     transfer_files: List[str] = field(default_factory=list)
     transfer_verify_dtr: bool = True
+    transfer_verify_package_name: bool = True
 
     priv_keys_model: KeyValueListModel = field(default_factory=KeyValueListModel)
     pub_keys_model: KeyValueListModel = field(default_factory=KeyValueListModel)
@@ -150,7 +179,7 @@ class AppData(ClassWithListener):
 
     validation_authority_key: Optional[gpg.Key] = None
 
-    def __post_init__(self):
+    def __post_init__(self) -> None:
         super().__init__()
         if self.config.offline:
             self.encrypt_verify_dtr = self.transfer_verify_dtr = False
@@ -161,3 +190,4 @@ class AppData(ClassWithListener):
             self.encrypt_compression_level = self.config.compression_level
         if self.config.package_name_suffix is not None:
             self.encrypt_package_name_suffix = self.config.package_name_suffix
+        self.transfer_verify_package_name = self.config.verify_package_name
