@@ -1,8 +1,11 @@
 from typing import Type, Dict, Tuple
 
-from .protocol import Protocol
+from .protocol import Protocol as _Protocol
 from . import liquid_files, sftp
 
+# reexport
+Protocol = _Protocol
+
 protocols: Tuple[Type[Protocol], ...] = (sftp.Protocol, liquid_files.Protocol)
 protocols_by_name: Dict[str, Type[Protocol]] = {
     p.__module__.replace(__name__ + ".", ""): p for p in protocols
